@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: binhex.c,v 1.10 1998/01/08 21:11:27 rob Exp $
+ * $Id: binhex.c,v 1.10 1998/04/11 20:51:56 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -368,7 +368,8 @@ int bh_open(int fd)
 
   /* find hqx header */
 
-  for (ptr = hqxheader; ptr - hqxheader < HEADERMATCH; )
+  ptr = hqxheader;
+  while (ptr == 0 || ptr - hqxheader < HEADERMATCH)
     {
       c = getc(file);
       if (c == EOF)
@@ -379,8 +380,14 @@ int bh_open(int fd)
 	  return -1;
 	}
 
-      if (c != *ptr++)
-	ptr = hqxheader;
+      if (c == '\n' || c == '\r')
+	{
+	  ptr = hqxheader;
+	  continue;
+	}
+
+      if (ptr && c != *ptr++)
+	ptr = 0;
     }
 
   /* skip to CR/LF */
