@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: block.c,v 1.9 1998/09/01 19:59:37 rob Exp $
+ * $Id: block.c,v 1.10 1998/09/15 19:21:03 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -190,13 +190,9 @@ int fillchain(hfsvol *vol, bucket **bptr, unsigned int *count)
   else
     {
       block buffer[HFS_BLOCKBUFSZ];
-      int result;
 
-      result = b_readpb(vol, vol->vstart + blist[0]->bnum, buffer, len);
-      if (result == -1)
+      if (b_readpb(vol, vol->vstart + blist[0]->bnum, buffer, len) == -1)
 	goto fail;
-      else if (result != len)
-	ERROR(EIO, "failed to read all consecutive blocks");
 
       for (i = 0; i < len; ++i)
 	memcpy(blist[i]->data, buffer[i], HFS_BLOCKSZ);
@@ -251,16 +247,12 @@ int flushchain(hfsvol *vol, bucket **bptr, unsigned int *count)
   else
     {
       block buffer[HFS_BLOCKBUFSZ];
-      int result;
 
       for (i = 0; i < len; ++i)
 	memcpy(buffer[i], blist[i]->data, HFS_BLOCKSZ);
 
-      result = b_writepb(vol, vol->vstart + blist[0]->bnum, buffer, len);
-      if (result == -1)
+      if (b_writepb(vol, vol->vstart + blist[0]->bnum, buffer, len) == -1)
 	goto fail;
-      else if (result != len)
-	ERROR(EIO, "failed to write all consecutive blocks");
     }
 
   for (i = 0; i < len; ++i)
@@ -581,11 +573,11 @@ fail:
 
 /*
  * NAME:	block->readpb()
- * DESCRIPTION:	read block(s) from the physical medium (N.B. bypasses cache)
+ * DESCRIPTION:	read blocks from the physical medium (bypassing cache)
  */
 int b_readpb(hfsvol *vol, unsigned long bnum, block *bp, unsigned int blen)
 {
-  long nbytes;
+  unsigned long nblocks;
 
 # ifdef DEBUG
   fprintf(stderr, "BLOCK: READ vol 0x%lx block %lu",
@@ -596,22 +588,21 @@ int b_readpb(hfsvol *vol, unsigned long bnum, block *bp, unsigned int blen)
     fprintf(stderr, "\n");
 # endif
 
-  if (blen == 0)
-    return 0;
-
-  if (os_seek(&vol->priv, bnum << HFS_BLOCKSZ_BITS, HFS_SEEK_SET) == -1)
+  nblocks = os_seek(&vol->priv, bnum);
+  if (nblocks == -1)
     goto fail;
 
-  nbytes = os_read(&vol->priv, bp, blen << HFS_BLOCKSZ_BITS);
-  if (nbytes == -1)
+  if (nblocks != bnum)
+    ERROR(EIO, "block seek failed for read");
+
+  nblocks = os_read(&vol->priv, bp, blen);
+  if (nblocks == -1)
     goto fail;
 
-  if (nbytes == 0)
-    ERROR(EIO, "read EOF on volume");
-  else if (nbytes < HFS_BLOCKSZ)
-    ERROR(EIO, "read incomplete block");
+  if (nblocks != blen)
+    ERROR(EIO, "incomplete block read");
 
-  return nbytes >> HFS_BLOCKSZ_BITS;
+  return 0;
 
 fail:
   return -1;
@@ -619,12 +610,12 @@ fail:
 
 /*
  * NAME:	block->writepb()
- * DESCRIPTION:	write a block to the physical medium (N.B. bypasses cache)
+ * DESCRIPTION:	write blocks to the physical medium (bypassing cache)
  */
 int b_writepb(hfsvol *vol, unsigned long bnum, const block *bp,
 	      unsigned int blen)
 {
-  long nbytes;
+  unsigned long nblocks;
 
 # ifdef DEBUG
   fprintf(stderr, "BLOCK: WRITE vol 0x%lx block %lu",
@@ -635,20 +626,21 @@ int b_writepb(hfsvol *vol, unsigned long bnum, const block *bp,
     fprintf(stderr, "\n");
 # endif
 
-  if (blen == 0)
-    return 0;
-
-  if (os_seek(&vol->priv, bnum << HFS_BLOCKSZ_BITS, HFS_SEEK_SET) == -1)
+  nblocks = os_seek(&vol->priv, bnum);
+  if (nblocks == -1)
     goto fail;
 
-  nbytes = os_write(&vol->priv, bp, blen << HFS_BLOCKSZ_BITS);
-  if (nbytes == -1)
+  if (nblocks != bnum)
+    ERROR(EIO, "block seek failed for write");
+
+  nblocks = os_write(&vol->priv, bp, blen);
+  if (nblocks == -1)
     goto fail;
 
-  if (nbytes < HFS_BLOCKSZ)
-    ERROR(EIO, "wrote incomplete block");
+  if (nblocks != blen)
+    ERROR(EIO, "incomplete block write");
 
-  return nbytes >> HFS_BLOCKSZ_BITS;
+  return 0;
 
 fail:
   return -1;
@@ -660,26 +652,25 @@ fail:
  */
 int b_readlb(hfsvol *vol, unsigned long bnum, block *bp)
 {
-  bucket *b;
-
   if (vol->vlen > 0 && bnum >= vol->vlen)
     ERROR(EIO, "read nonexistent logical block");
 
-  if (vol->cache == 0)
+  if (vol->cache)
+    {
+      bucket *b;
+
+      b = getbucket(vol->cache, bnum, 1);
+      if (b == 0)
+	goto fail;
+
+      memcpy(bp, b->data, HFS_BLOCKSZ);
+    }
+  else
     {
       if (b_readpb(vol, vol->vstart + bnum, bp, 1) == -1)
 	goto fail;
-
-      goto done;
     }
 
-  b = getbucket(vol->cache, bnum, 1);
-  if (b == 0)
-    goto fail;
-
-  memcpy(bp, b->data, HFS_BLOCKSZ);
-
-done:
   return 0;
 
 fail:
@@ -692,31 +683,30 @@ fail:
  */
 int b_writelb(hfsvol *vol, unsigned long bnum, const block *bp)
 {
-  bucket *b;
-
   if (vol->vlen > 0 && bnum >= vol->vlen)
     ERROR(EIO, "write nonexistent logical block");
 
-  if (vol->cache == 0)
+  if (vol->cache)
+    {
+      bucket *b;
+
+      b = getbucket(vol->cache, bnum, 0);
+      if (b == 0)
+	goto fail;
+
+      if (! INUSE(b) ||
+	  memcmp(b->data, bp, HFS_BLOCKSZ) != 0)
+	{
+	  memcpy(b->data, bp, HFS_BLOCKSZ);
+	  b->flags |= HFS_BUCKET_INUSE | HFS_BUCKET_DIRTY;
+	}
+    }
+  else
     {
       if (b_writepb(vol, vol->vstart + bnum, bp, 1) == -1)
 	goto fail;
-
-      goto done;
     }
 
-  b = getbucket(vol->cache, bnum, 0);
-  if (b == 0)
-    goto fail;
-
-  if (! INUSE(b) ||
-      memcmp(b->data, bp, HFS_BLOCKSZ) != 0)
-    {
-      memcpy(b->data, bp, HFS_BLOCKSZ);
-      b->flags |= HFS_BUCKET_INUSE | HFS_BUCKET_DIRTY;
-    }
-
-done:
   return 0;
 
 fail:
@@ -774,14 +764,13 @@ fail:
  */
 unsigned long b_size(hfsvol *vol)
 {
-  long nbytes;
   unsigned long low, high, mid;
   block b;
 
-  nbytes = os_seek(&vol->priv, 0, HFS_SEEK_END);
+  high = os_seek(&vol->priv, -1);
 
-  if (nbytes != -1 && (unsigned long) nbytes >= HFS_BLOCKSZ)
-    return (unsigned long) nbytes >> HFS_BLOCKSZ_BITS;
+  if (high != -1 && high > 0)
+    return high;
 
   /* manual size detection: first check there is at least 1 block in medium */
 
