@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: btree.c,v 1.8 1998/09/08 19:07:57 rob Exp $
+ * $Id: btree.c,v 1.9 1998/09/10 04:56:06 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -38,13 +38,15 @@
  * NAME:	btree->getnode()
  * DESCRIPTION:	retrieve a numbered node from a B*-tree file
  */
-int bt_getnode(node *np)
+int bt_getnode(node *np, btree *bt, unsigned long nnum)
 {
-  btree *bt = np->bt;
   block *bp = &np->data;
   const byte *ptr;
   int i;
 
+  np->bt   = bt;
+  np->nnum = nnum;
+
 # if 0
   fprintf(stderr, "BTREE: GET vol \"%s\" btree \"%s\" node %lu\n",
 	  bt->f.vol->mdb.drVN, bt->f.name, np->nnum);
@@ -52,12 +54,12 @@ int bt_getnode(node *np)
 
   /* verify the node exists and is marked as in-use */
 
-  if (np->nnum > 0 && np->nnum >= bt->hdr.bthNNodes)
+  if (nnum > 0 && nnum >= bt->hdr.bthNNodes)
     ERROR(EIO, "read nonexistent b*-tree node");
-  else if (bt->map && ! BMTST(bt->map, np->nnum))
+  else if (bt->map && ! BMTST(bt->map, nnum))
     ERROR(EIO, "read unallocated b*-tree node");
 
-  if (f_getblock(&bt->f, np->nnum, bp) == -1)
+  if (f_getblock(&bt->f, nnum, bp) == -1)
     goto fail;
 
   ptr = *bp;
@@ -144,10 +146,7 @@ int bt_readhdr(btree *bt)
   int i;
   unsigned long nnum;
 
-  bt->hdrnd.bt   = bt;
-  bt->hdrnd.nnum = 0;
-
-  if (bt_getnode(&bt->hdrnd) == -1)
+  if (bt_getnode(&bt->hdrnd, bt, 0) == -1)
     goto fail;
 
   if (bt->hdrnd.nd.ndType != ndHdrNode ||
@@ -197,10 +196,7 @@ int bt_readhdr(btree *bt)
       node n;
       byte *newmap;
 
-      n.bt   = bt;
-      n.nnum = nnum;
-
-      if (bt_getnode(&n) == -1)
+      if (bt_getnode(&n, bt, nnum) == -1)
 	goto fail;
 
       if (n.nd.ndType != ndMapNode ||
@@ -277,10 +273,7 @@ int bt_writehdr(btree *bt)
       if (nnum == 0)
 	ERROR(EIO, "truncated b*-tree map");
 
-      n.bt   = bt;
-      n.nnum = nnum;
-
-      if (bt_getnode(&n) == -1)
+      if (bt_getnode(&n, bt, nnum) == -1)
 	goto fail;
 
       if (n.nd.ndType != ndMapNode ||
@@ -365,6 +358,9 @@ int bt_space(btree *bt, unsigned int nrecs)
       if (n_new(&mapnd) == -1)
 	goto fail;
 
+      mapnd.nd.ndNRecs = 1;
+      mapnd.roff[1]    = 0x1fa;
+
       /* link the new map node */
 
       if (bt->hdrnd.nd.ndFLink == 0)
@@ -375,19 +371,19 @@ int bt_space(btree *bt, unsigned int nrecs)
       else
 	{
 	  node n;
+	  unsigned long nnum;
 
-	  n.bt   = bt;
-	  n.nnum = bt->hdrnd.nd.ndFLink;
+	  nnum = bt->hdrnd.nd.ndFLink;
 
 	  while (1)
 	    {
-	      if (bt_getnode(&n) == -1)
+	      if (bt_getnode(&n, bt, nnum) == -1)
 		goto fail;
 
 	      if (n.nd.ndFLink == 0)
 		break;
 
-	      n.nnum = n.nd.ndFLink;
+	      nnum = n.nd.ndFLink;
 	    }
 
 	  n.nd.ndFLink     = mapnd.nnum;
@@ -397,9 +393,6 @@ int bt_space(btree *bt, unsigned int nrecs)
 	    goto fail;
 	}
 
-      mapnd.nd.ndNRecs = 1;
-      mapnd.roff[1]    = 0x1fa;
-
       if (bt_putnode(&mapnd) == -1)
 	goto fail;
     }
@@ -433,10 +426,7 @@ int insertx(node *np, byte *record, int *reclen)
       else
 	rec = HFS_NODEREC(*np, np->rnum);
 
-      child.bt   = np->bt;
-      child.nnum = d_getul(HFS_RECDATA(rec));
-
-      if (bt_getnode(&child) == -1 ||
+      if (bt_getnode(&child, np->bt, d_getul(HFS_RECDATA(rec))) == -1 ||
 	  insertx(&child, record, reclen) == -1)
 	goto fail;
 
@@ -495,14 +485,8 @@ int bt_insert(btree *bt, const byte *record, int reclen)
 
       bt->flags |= HFS_BT_UPDATE_HDR;
     }
-  else
-    {
-      root.bt   = bt;
-      root.nnum = bt->hdr.bthRoot;
-
-      if (bt_getnode(&root) == -1)
-	goto fail;
-    }
+  else if (bt_getnode(&root, bt, bt->hdr.bthRoot) == -1)
+    goto fail;
 
   memcpy(newrec, record, reclen);
 
@@ -569,10 +553,7 @@ int deletex(node *np, const byte *key, byte *record, int *flag)
 
       rec = HFS_NODEREC(*np, np->rnum);
 
-      child.bt   = np->bt;
-      child.nnum = d_getul(HFS_RECDATA(rec));
-
-      if (bt_getnode(&child) == -1 ||
+      if (bt_getnode(&child, np->bt, d_getul(HFS_RECDATA(rec))) == -1 ||
 	  deletex(&child, key, rec, flag) == -1)
 	goto fail;
 
@@ -626,13 +607,10 @@ int bt_delete(btree *bt, const byte *key)
   byte record[HFS_MAX_RECLEN];
   int flag = 0;
 
-  root.bt   = bt;
-  root.nnum = bt->hdr.bthRoot;
-
-  if (root.nnum == 0)
+  if (bt->hdr.bthRoot == 0)
     ERROR(EIO, "empty b*-tree");
 
-  if (bt_getnode(&root) == -1 ||
+  if (bt_getnode(&root, bt, bt->hdr.bthRoot) == -1 ||
       deletex(&root, key, record, &flag) == -1)
     goto fail;
 
@@ -640,7 +618,7 @@ int bt_delete(btree *bt, const byte *key)
     {
       const byte *rec;
 
-      /* chop the root */
+      /* root only has one record; eliminate it and decrease the tree depth */
 
       rec = HFS_NODEREC(root, 0);
 
@@ -674,18 +652,18 @@ fail:
 int bt_search(btree *bt, const byte *key, node *np)
 {
   int found = 0;
+  unsigned long nnum;
 
-  np->bt   = bt;
-  np->nnum = bt->hdr.bthRoot;
+  nnum = bt->hdr.bthRoot;
 
-  if (np->nnum == 0)
+  if (nnum == 0)
     ERROR(ENOENT, 0);
 
   while (1)
     {
       const byte *rec;
 
-      if (bt_getnode(np) == -1)
+      if (bt_getnode(np, bt, nnum) == -1)
 	{
 	  found = -1;
 	  goto fail;
@@ -699,8 +677,9 @@ int bt_search(btree *bt, const byte *key, node *np)
 	  if (np->rnum == -1)
 	    ERROR(ENOENT, 0);
 
-	  rec = HFS_NODEREC(*np, np->rnum);
-	  np->nnum = d_getul(HFS_RECDATA(rec));
+	  rec  = HFS_NODEREC(*np, np->rnum);
+	  nnum = d_getul(HFS_RECDATA(rec));
+
 	  break;
 
 	case ndLeafNode:
