@@ -16,10 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: btree.h,v 1.6 1998/04/11 08:27:11 rob Exp $
+ * $Id: btree.h,v 1.7 1998/09/10 04:56:07 rob Exp $
  */
 
-int bt_getnode(node *);
+int bt_getnode(node *, btree *, unsigned long);
 int bt_putnode(node *);
 
 int bt_readhdr(btree *);
