@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: file.c,v 1.6 1998/04/11 08:27:12 rob Exp $
+ * $Id: file.c,v 1.8 1998/09/17 01:04:49 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -316,7 +316,14 @@ long f_alloc(hfsfile *file)
 
   clumpsz = file->cat.u.fil.filClpSize;
   if (clumpsz == 0)
-    clumpsz = vol->mdb.drClpSiz;
+    {
+      if (file == &vol->ext.f)
+	clumpsz = vol->mdb.drXTClpSiz;
+      else if (file == &vol->cat.f)
+	clumpsz = vol->mdb.drCTClpSiz;
+      else
+	clumpsz = vol->mdb.drClpSiz;
+    }
 
   blocks.xdrNumABlks = clumpsz / vol->mdb.drAlBlkSiz;
 
@@ -492,8 +499,8 @@ int f_flush(hfsfile *file)
     {
       node n;
 
-      file->cat.u.fil.filStBlk   = file->cat.u.fil.filExtRec[0].xdrStABN;
-      file->cat.u.fil.filRStBlk  = file->cat.u.fil.filRExtRec[0].xdrStABN;
+      file->cat.u.fil.filStBlk  = file->cat.u.fil.filExtRec[0].xdrStABN;
+      file->cat.u.fil.filRStBlk = file->cat.u.fil.filRExtRec[0].xdrStABN;
 
       if (v_catsearch(vol, file->parid, file->name, 0, 0, &n) <= 0 ||
 	  v_putcatrec(&file->cat, &n) == -1)
