@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: hfs.c,v 1.10 1998/08/31 04:27:26 rob Exp $
+ * $Id: hfs.c,v 1.14 1998/09/17 01:04:50 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -574,14 +574,13 @@ int hfs_readdir(hfsdir *dir, hfsdirent *ent)
 
       while (dir->n.rnum >= dir->n.nd.ndNRecs)
 	{
-	  dir->n.nnum = dir->n.nd.ndFLink;
-	  if (dir->n.nnum == 0)
+	  if (dir->n.nd.ndFLink == 0)
 	    {
 	      dir->n.rnum = -1;
 	      ERROR(ENOENT, "no more entries");
 	    }
 
-	  if (bt_getnode(&dir->n) == -1)
+	  if (bt_getnode(&dir->n, dir->n.bt, dir->n.nd.ndFLink) == -1)
 	    {
 	      dir->n.rnum = -1;
 	      goto fail;
@@ -656,11 +655,11 @@ hfsfile *hfs_create(hfsvol *vol, const char *path,
 		    const char *type, const char *creator)
 {
   hfsfile *file = 0;
-  long id;
-  CatDataRec *cat;
+  long parid;
+  char name[HFS_MAX_FLEN + 1];
   CatKeyRec key;
   byte record[HFS_MAX_CATRECLEN];
-  int found, i, reclen;
+  int found, reclen;
 
   if (getvol(&vol) == -1)
     goto fail;
@@ -669,70 +668,41 @@ hfsfile *hfs_create(hfsvol *vol, const char *path,
   if (file == 0)
     ERROR(ENOMEM, 0);
 
-  cat = &file->cat;
-
-  found = v_resolve(&vol, path, cat, &file->parid, file->name, 0);
-  if (found == -1 || file->parid == 0)
+  found = v_resolve(&vol, path, &file->cat, &parid, name, 0);
+  if (found == -1 || parid == 0)
     goto fail;
 
   if (found)
     ERROR(EEXIST, 0);
 
-  if (file->parid == HFS_CNID_ROOTPAR)
+  if (parid == HFS_CNID_ROOTPAR)
     ERROR(EINVAL, 0);
 
   if (vol->flags & HFS_VOL_READONLY)
     ERROR(EROFS, 0);
 
-  /* create file `file->name' in parent `file->parid' */
+  /* create file `name' in parent `parid' */
 
   if (bt_space(&vol->cat, 1) == -1)
     goto fail;
 
-  id = vol->mdb.drNxtCNID++;
+  f_init(file, vol, vol->mdb.drNxtCNID++, name);
   vol->flags |= HFS_VOL_UPDATE_MDB;
 
+  file->parid = parid;
+
   /* create catalog record */
 
-  cat->cdrType   = cdrFilRec;
-  cat->cdrResrv2 = 0;
+  file->cat.u.fil.filUsrWds.fdType =
+    d_getsl((const unsigned char *) type);
+  file->cat.u.fil.filUsrWds.fdCreator =
+    d_getsl((const unsigned char *) creator);
 
-  cat->u.fil.filFlags = 0;
-  cat->u.fil.filTyp   = 0;
-
-  memset(&cat->u.fil.filUsrWds, 0, sizeof(cat->u.fil.filUsrWds));
-
-  cat->u.fil.filUsrWds.fdType    = d_getsl((const unsigned char *) type);
-  cat->u.fil.filUsrWds.fdCreator = d_getsl((const unsigned char *) creator);
-
-  cat->u.fil.filFlNum  = id;
-  cat->u.fil.filStBlk  = 0;
-  cat->u.fil.filLgLen  = 0;
-  cat->u.fil.filPyLen  = 0;
-  cat->u.fil.filRStBlk = 0;
-  cat->u.fil.filRLgLen = 0;
-  cat->u.fil.filRPyLen = 0;
-  cat->u.fil.filCrDat  = d_mtime(time(0));
-  cat->u.fil.filMdDat  = cat->u.fil.filCrDat;
-  cat->u.fil.filBkDat  = 0;
-
-  memset(&cat->u.fil.filFndrInfo, 0, sizeof(cat->u.fil.filFndrInfo));
-
-  cat->u.fil.filClpSize = 0;
-
-  for (i = 0; i < 3; ++i)
-    {
-      cat->u.fil.filExtRec[i].xdrStABN     = 0;
-      cat->u.fil.filExtRec[i].xdrNumABlks  = 0;
-
-      cat->u.fil.filRExtRec[i].xdrStABN    = 0;
-      cat->u.fil.filRExtRec[i].xdrNumABlks = 0;
-    }
-
-  cat->u.fil.filResrv = 0;
+  file->cat.u.fil.filCrDat = d_mtime(time(0));
+  file->cat.u.fil.filMdDat = file->cat.u.fil.filCrDat;
 
   r_makecatkey(&key, file->parid, file->name);
-  r_packcatrec(&key, cat, record, &reclen);
+  r_packcatrec(&key, &file->cat, record, &reclen);
 
   if (bt_insert(&vol->cat, record, reclen) == -1 ||
       v_adjvalence(vol, file->parid, 0, 1) == -1)
@@ -740,12 +710,6 @@ hfsfile *hfs_create(hfsvol *vol, const char *path,
 
   /* package file handle for user */
 
-  file->vol   = vol;
-  file->flags = 0;
-
-  f_selectfork(file, fkData);
-
-  file->prev = 0;
   file->next = vol->files;
 
   if (vol->files)
@@ -1597,9 +1561,6 @@ int hfs_format(const char *path, int pnum, const char *vname,
       v_geometry(&vol, pnum) == -1)
     goto fail;
 
-  if (vol.vlen < 800 * (1024 >> HFS_BLOCKSZ_BITS))
-    ERROR(EINVAL, "volume size must be at least 800K");
-
   /* initialize volume geometry */
 
   vol.lpa = 1 + ((vol.vlen - 6) >> 16);
@@ -1625,8 +1586,8 @@ int hfs_format(const char *path, int pnum, const char *vname,
   vol.mdb.drVSeqNum  = 0;
   vol.mdb.drWrCnt    = 0;
 
-  /* vol.mdb.drXTClpSiz */
-  /* vol.mdb.drCTClpSiz */
+  vol.mdb.drXTClpSiz = vol.mdb.drNmAlBlks / 128 * vol.mdb.drAlBlkSiz;
+  vol.mdb.drCTClpSiz = vol.mdb.drXTClpSiz;
 
   vol.mdb.drNmRtDirs = 0;
   vol.mdb.drFilCnt   = 0;
@@ -1692,8 +1653,6 @@ int hfs_format(const char *path, int pnum, const char *vname,
 
   /* create extents overflow file */
 
-  ext->f.cat.u.fil.filClpSize = vol.mdb.drNmAlBlks / 128 * vol.mdb.drAlBlkSiz;
-
   n_init(&ext->hdrnd, ext, ndHdrNode, 0);
 
   ext->hdrnd.nnum       = 0;
@@ -1723,13 +1682,11 @@ int hfs_format(const char *path, int pnum, const char *vname,
   memset(ext->map, 0, HFS_MAP1SZ);
   BMSET(ext->map, 0);
 
-  ext->mapsz   = HFS_MAP1SZ;
-  ext->flags   = HFS_BT_UPDATE_HDR;
+  ext->mapsz = HFS_MAP1SZ;
+  ext->flags = HFS_BT_UPDATE_HDR;
 
   /* create catalog file */
 
-  cat->f.cat.u.fil.filClpSize = ext->f.cat.u.fil.filClpSize;
-
   n_init(&cat->hdrnd, cat, ndHdrNode, 0);
 
   cat->hdrnd.nnum       = 0;
@@ -1759,8 +1716,8 @@ int hfs_format(const char *path, int pnum, const char *vname,
   memset(cat->map, 0, HFS_MAP1SZ);
   BMSET(cat->map, 0);
 
-  cat->mapsz   = HFS_MAP1SZ;
-  cat->flags   = HFS_BT_UPDATE_HDR;
+  cat->mapsz = HFS_MAP1SZ;
+  cat->flags = HFS_BT_UPDATE_HDR;
 
   /* allocate space for header nodes (and initial extents) */
 
