@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: node.c,v 1.7 1998/09/08 18:53:06 rob Exp $
+ * $Id: node.c,v 1.8 1998/09/10 04:56:10 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -113,10 +113,7 @@ int n_free(node *np)
 
   if (np->nd.ndFLink > 0)
     {
-      sib.bt   = bt;
-      sib.nnum = np->nd.ndFLink;
-
-      if (bt_getnode(&sib) == -1)
+      if (bt_getnode(&sib, bt, np->nd.ndFLink) == -1)
 	goto fail;
 
       sib.nd.ndBLink = np->nd.ndBLink;
@@ -127,10 +124,7 @@ int n_free(node *np)
 
   if (np->nd.ndBLink > 0)
     {
-      sib.bt   = bt;
-      sib.nnum = np->nd.ndBLink;
-
-      if (bt_getnode(&sib) == -1)
+      if (bt_getnode(&sib, bt, np->nd.ndBLink) == -1)
 	goto fail;
 
       sib.nd.ndFLink = np->nd.ndFLink;
@@ -329,10 +323,7 @@ int split(node *left, byte *record, int *reclen)
     {
       node sib;
 
-      sib.bt   = right->bt;
-      sib.nnum = right->nd.ndFLink;
-
-      if (bt_getnode(&sib) == -1)
+      if (bt_getnode(&sib, right->bt, right->nd.ndFLink) == -1)
 	goto fail;
 
       sib.nd.ndBLink = right->nnum;
@@ -458,10 +449,7 @@ int n_delete(node *np, byte *record, int *flag)
     {
       node left;
 
-      left.bt   = np->bt;
-      left.nnum = np->nd.ndBLink;
-
-      if (bt_getnode(&left) == -1)
+      if (bt_getnode(&left, np->bt, np->nd.ndBLink) == -1)
 	goto fail;
 
       if (np->nd.ndNRecs + left.nd.ndNRecs <= HFS_MAX_NRECS &&
