@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: unix.c,v 1.5 1998/04/11 08:27:16 rob Exp $
+ * $Id: unix.c,v 1.7 1998/09/15 19:21:21 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -135,34 +135,24 @@ fail:
 
 /*
  * NAME:	os->seek()
- * DESCRIPTION:	set a descriptor's seek pointer
+ * DESCRIPTION:	set a descriptor's seek pointer (offset in blocks)
  */
-long os_seek(void **priv, long offset, int from)
+unsigned long os_seek(void **priv, unsigned long offset)
 {
-  long result;
+  int fd = (int) *priv;
+  unsigned long result;
 
-  switch (from)
-    {
-    case HFS_SEEK_CUR:
-      from = SEEK_CUR;
-      break;
+  /* offset == -1 special; seek to last block of device */
 
-    case HFS_SEEK_END:
-      from = SEEK_END;
-      break;
-
-    case HFS_SEEK_SET:
-    default:
-      from = SEEK_SET;
-      break;
-    }
-
-  result = lseek((int) *priv, offset, from);
+  if (offset == -1)
+    result = lseek(fd, 0, SEEK_END);
+  else
+    result = lseek(fd, offset << HFS_BLOCKSZ_BITS, SEEK_SET);
 
   if (result == -1)
     ERROR(errno, "error seeking medium");
 
-  return result;
+  return result >> HFS_BLOCKSZ_BITS;
 
 fail:
   return -1;
@@ -170,18 +160,19 @@ fail:
 
 /*
  * NAME:	os->read()
- * DESCRIPTION:	read bytes from an open descriptor
+ * DESCRIPTION:	read blocks from an open descriptor
  */
-long os_read(void **priv, void *buf, unsigned long len)
+unsigned long os_read(void **priv, void *buf, unsigned long len)
 {
-  long result;
+  int fd = (int) *priv;
+  unsigned long result;
 
-  result = read((int) *priv, buf, len);
+  result = read(fd, buf, len << HFS_BLOCKSZ_BITS);
 
   if (result == -1)
     ERROR(errno, "error reading from medium");
 
-  return result;
+  return result >> HFS_BLOCKSZ_BITS;
 
 fail:
   return -1;
@@ -189,18 +180,19 @@ fail:
 
 /*
  * NAME:	os->write()
- * DESCRIPTION:	write bytes to an open descriptor
+ * DESCRIPTION:	write blocks to an open descriptor
  */
-long os_write(void **priv, const void *buf, unsigned long len)
+unsigned long os_write(void **priv, const void *buf, unsigned long len)
 {
-  long result;
+  int fd = (int) *priv;
+  unsigned long result;
 
-  result = write((int) *priv, buf, len);
+  result = write(fd, buf, len << HFS_BLOCKSZ_BITS);
 
   if (result == -1)
     ERROR(errno, "error writing to medium");
 
-  return result;
+  return result >> HFS_BLOCKSZ_BITS;
 
 fail:
   return -1;
