@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: volume.c,v 1.8 1998/04/11 08:27:15 rob Exp $
+ * $Id: volume.c,v 1.11 1998/09/17 00:13:29 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -246,34 +246,36 @@ int v_geometry(hfsvol *vol, int pnum)
 
       if (vol->vlen == 0)
 	goto fail;
-
-      goto done;
     }
-
-  while (pnum--)
+  else
     {
-      found = m_findpmentry(vol, "Apple_HFS", &map, &bnum);
-      if (found == -1 || ! found)
-	goto fail;
+      while (pnum--)
+	{
+	  found = m_findpmentry(vol, "Apple_HFS", &map, &bnum);
+	  if (found == -1 || ! found)
+	    goto fail;
+	}
+
+      vol->vstart = map.pmPyPartStart;
+      vol->vlen   = map.pmPartBlkCnt;
+
+      if (map.pmDataCnt)
+	{
+	  if ((unsigned long) map.pmLgDataStart +
+	      (unsigned long) map.pmDataCnt > vol->vlen)
+	    ERROR(EINVAL, "partition data overflows partition");
+
+	  vol->vstart += (unsigned long) map.pmLgDataStart;
+	  vol->vlen    = map.pmDataCnt;
+	}
+
+      if (vol->vlen == 0)
+	ERROR(EINVAL, "volume partition is empty");
     }
 
-  vol->vstart = map.pmPyPartStart;
-  vol->vlen   = map.pmPartBlkCnt;
+  if (vol->vlen < 800 * (1024 >> HFS_BLOCKSZ_BITS))
+    ERROR(EINVAL, "volume is smaller than 800K");
 
-  if (map.pmDataCnt)
-    {
-      if ((unsigned long) map.pmLgDataStart +
-	  (unsigned long) map.pmDataCnt > vol->vlen)
-	ERROR(EINVAL, "partition data overflows partition");
-
-      vol->vstart += (unsigned long) map.pmLgDataStart;
-      vol->vlen    = map.pmDataCnt;
-    }
-
-  if (vol->vlen == 0)
-    ERROR(EINVAL, "volume partition is empty");
-
-done:
   return 0;
 
 fail:
@@ -304,14 +306,12 @@ int v_readmdb(hfsvol *vol)
 
   /* extents pseudo-file structs */
 
-  vol->ext.f.cat.u.fil.filStBlk   = vol->mdb.drXTExtRec[0].xdrStABN;
-  vol->ext.f.cat.u.fil.filLgLen   = vol->mdb.drXTFlSize;
-  vol->ext.f.cat.u.fil.filPyLen   = vol->mdb.drXTFlSize;
+  vol->ext.f.cat.u.fil.filStBlk = vol->mdb.drXTExtRec[0].xdrStABN;
+  vol->ext.f.cat.u.fil.filLgLen = vol->mdb.drXTFlSize;
+  vol->ext.f.cat.u.fil.filPyLen = vol->mdb.drXTFlSize;
 
-  vol->ext.f.cat.u.fil.filCrDat   = vol->mdb.drCrDate;
-  vol->ext.f.cat.u.fil.filMdDat   = vol->mdb.drLsMod;
-
-  vol->ext.f.cat.u.fil.filClpSize = vol->mdb.drXTClpSiz;
+  vol->ext.f.cat.u.fil.filCrDat = vol->mdb.drCrDate;
+  vol->ext.f.cat.u.fil.filMdDat = vol->mdb.drLsMod;
 
   memcpy(&vol->ext.f.cat.u.fil.filExtRec,
 	 &vol->mdb.drXTExtRec, sizeof(ExtDataRec));
@@ -320,14 +320,12 @@ int v_readmdb(hfsvol *vol)
 
   /* catalog pseudo-file structs */
 
-  vol->cat.f.cat.u.fil.filStBlk   = vol->mdb.drCTExtRec[0].xdrStABN;
-  vol->cat.f.cat.u.fil.filLgLen   = vol->mdb.drCTFlSize;
-  vol->cat.f.cat.u.fil.filPyLen   = vol->mdb.drCTFlSize;
+  vol->cat.f.cat.u.fil.filStBlk = vol->mdb.drCTExtRec[0].xdrStABN;
+  vol->cat.f.cat.u.fil.filLgLen = vol->mdb.drCTFlSize;
+  vol->cat.f.cat.u.fil.filPyLen = vol->mdb.drCTFlSize;
 
-  vol->cat.f.cat.u.fil.filCrDat   = vol->mdb.drCrDate;
-  vol->cat.f.cat.u.fil.filMdDat   = vol->mdb.drLsMod;
-
-  vol->cat.f.cat.u.fil.filClpSize = vol->mdb.drCTClpSiz;
+  vol->cat.f.cat.u.fil.filCrDat = vol->mdb.drCrDate;
+  vol->cat.f.cat.u.fil.filMdDat = vol->mdb.drLsMod;
 
   memcpy(&vol->cat.f.cat.u.fil.filExtRec,
 	 &vol->mdb.drCTExtRec, sizeof(ExtDataRec));
@@ -347,13 +345,11 @@ fail:
 int v_writemdb(hfsvol *vol)
 {
   vol->mdb.drXTFlSize = vol->ext.f.cat.u.fil.filPyLen;
-  vol->mdb.drXTClpSiz = vol->ext.f.cat.u.fil.filClpSize;
 
   memcpy(&vol->mdb.drXTExtRec,
 	 &vol->ext.f.cat.u.fil.filExtRec, sizeof(ExtDataRec));
 
   vol->mdb.drCTFlSize = vol->cat.f.cat.u.fil.filPyLen;
-  vol->mdb.drCTClpSiz = vol->cat.f.cat.u.fil.filClpSize;
 
   memcpy(&vol->mdb.drCTExtRec,
 	 &vol->cat.f.cat.u.fil.filExtRec, sizeof(ExtDataRec));
@@ -1021,12 +1017,9 @@ int v_scavenge(hfsvol *vol)
 
   /* scavenge the extents overflow file */
 
-  n.bt   = &vol->ext;
-  n.nnum = vol->ext.hdr.bthFNode;
-
-  if (n.nnum > 0)
+  if (vol->ext.hdr.bthFNode > 0)
     {
-      if (bt_getnode(&n) == -1)
+      if (bt_getnode(&n, &vol->ext, vol->ext.hdr.bthFNode) == -1)
 	goto fail;
 
       n.rnum = 0;
@@ -1038,17 +1031,16 @@ int v_scavenge(hfsvol *vol)
 
 	  while (n.rnum >= n.nd.ndNRecs)
 	    {
-	      n.nnum = n.nd.ndFLink;
-	      if (n.nnum == 0)
+	      if (n.nd.ndFLink == 0)
 		break;
 
-	      if (bt_getnode(&n) == -1)
+	      if (bt_getnode(&n, &vol->ext, n.nd.ndFLink) == -1)
 		goto fail;
 
 	      n.rnum = 0;
 	    }
 
-	  if (n.nnum == 0)
+	  if (n.nd.ndFLink == 0)
 	    break;
 
 	  ptr = HFS_NODEREC(n, n.rnum);
@@ -1062,12 +1054,9 @@ int v_scavenge(hfsvol *vol)
 
   /* scavenge the catalog file */
 
-  n.bt   = &vol->cat;
-  n.nnum = vol->cat.hdr.bthFNode;
-
-  if (n.nnum > 0)
+  if (vol->cat.hdr.bthFNode > 0)
     {
-      if (bt_getnode(&n) == -1)
+      if (bt_getnode(&n, &vol->cat, vol->cat.hdr.bthFNode) == -1)
 	goto fail;
 
       n.rnum = 0;
@@ -1079,17 +1068,16 @@ int v_scavenge(hfsvol *vol)
 
 	  while (n.rnum >= n.nd.ndNRecs)
 	    {
-	      n.nnum = n.nd.ndFLink;
-	      if (n.nnum == 0)
+	      if (n.nd.ndFLink == 0)
 		break;
 
-	      if (bt_getnode(&n) == -1)
+	      if (bt_getnode(&n, &vol->cat, n.nd.ndFLink) == -1)
 		goto fail;
 
 	      n.rnum = 0;
 	    }
 
-	  if (n.nnum == 0)
+	  if (n.nd.ndFLink == 0)
 	    break;
 
 	  ptr = HFS_NODEREC(n, n.rnum);
